#include <errno.h>
#include <gdbm.h>
#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/stat.h>

#include "chirp_client.h"
#include "chirp_reli.h"
#include "chirp_group.h"
#include "create_dir.h"
#include "debug.h"
#include "list.h"
#include "stringtools.h"


static struct policy current_group_policy = {{0}, 0, 0};
const char POLICY_DB[] = ".__cpol";
static char root_path[CHIRP_PATH_MAX];
static int  local_decision_duration = 20;

INT64_T local_timeout = 100;
struct list *cache_file_index;
struct list *dec_cache_list;
struct list *cache_policy_list;
                                                                                                                      
/********************************* GROUP MANIPULATION FUNCTIONS ***************************************/

/* Called by the Chirp server during initialization so chirp_group routines can access the Chirp root directory */
void chirp_group_set_dir(const char* path)
{
        strncpy(root_path, path, CHIRP_PATH_MAX);
}

static void chirp_group_create_cache_dir()
{
        char path[CHIRP_PATH_MAX];
	int flags = 0;
	
	flags |= (S_IRWXU | S_IXGRP | S_IRGRP | S_IXOTH | S_IROTH);
	
	sprintf(path, "%s/%s", root_path, CACHE_DIR);
	create_dir(path, flags);
}


INT64_T chirp_group_create(char* path)
{
        GDBM_FILE dbf;
	struct stat buf;

	string_collapse_path(path, path, 1);
	
	if (!stat(path, &buf))
	  {
	    errno = EEXIST;
	    return CHIRP_ERROR_ALREADY_EXISTS;
	  }

	dbf = gdbm_open (path, 0, GDBM_WRCREAT, 0644, 0);
	gdbm_close (dbf);

	return 1;
}

INT64_T chirp_group_set_policy(char* path, int file_cache_duration, int dec_cache_duration)
{
        char path_to_db[CHIRP_PATH_MAX];
	char policy[CHIRP_LINE_MAX];
	GDBM_FILE grp_policy_db;
	datum new_key, new_value;

	chirp_group_create_cache_dir();

	sprintf(path_to_db, "%s/%s/%s", root_path, CACHE_DIR, POLICY_DB);

	grp_policy_db = gdbm_open(path_to_db, 0, GDBM_WRCREAT, 0644, 0);

	if (!grp_policy_db)
	  return -1;

	sprintf(policy, "%d %d\n", file_cache_duration, dec_cache_duration);

	new_key.dsize = strlen(path) + 1;
	new_value.dsize = strlen(policy) + 1;
	new_key.dptr = (char*)malloc(sizeof(char) * new_key.dsize);
	new_value.dptr = (char*)malloc(sizeof(char) * new_value.dsize);

	if (!new_key.dptr || !new_value.dptr)
	  return -1;

	strncpy(new_key.dptr, path, new_key.dsize);
	strncpy(new_value.dptr, policy, new_value.dsize);
	gdbm_store(grp_policy_db, new_key, new_value, GDBM_REPLACE);
	gdbm_close(grp_policy_db);

	return 1;
}

/* Inserts a username in the appropriate group definition file if it doesn't already exist */

INT64_T chirp_group_add_user(char* path, char* usr_name)
{
        GDBM_FILE dbf;
	datum key, value;
	int found;
	struct stat buf;
	
	string_collapse_path(path, path, 1);
	
	if (stat(path, &buf) < 0) 
	  {
	    return -1;
	  }

	dbf = gdbm_open(path, 0, GDBM_WRITER, 0644, 0);
	if (!dbf)
	  {
	    errno = gdbm_errno;
	    return -1;
	  }
	
	key.dsize = strlen(usr_name) + 1;
	key.dptr = (char*)malloc(sizeof(char) * key.dsize);
	if (!key.dptr)
	  return -1;
	strncpy(key.dptr, usr_name, key.dsize);
	
	value.dsize = key.dsize;
	value.dptr = (char*)malloc(sizeof(char) * value.dsize);
	if (!value.dptr)
	  return -1;
	strncpy(value.dptr, usr_name, value.dsize);
	
	found = gdbm_store (dbf, key, value, GDBM_INSERT);
	gdbm_close(dbf);
	
	return (INT64_T) found;
}

/* Deletes a username to the appropriate group definition file if it exists.  It copies all other entries to a temp file, then unlinks the original and renames the temp file */

INT64_T chirp_group_delete_user(char* path, char* usr_name)
{
        struct stat buf;
	GDBM_FILE dbf;
	datum key;
	int found;
	
	string_collapse_path(path, path, 1);
	if (stat(path, &buf) < 0) 
	  {
	    return CHIRP_ERROR_DOESNT_EXIST;
	  }
	
	key.dsize = strlen(usr_name) + 1;
	key.dptr = (char*)malloc(sizeof(char) * key.dsize);
	if (!key.dptr)
	  return -1;
	strncpy(key.dptr, usr_name, key.dsize);
	
	dbf = gdbm_open(path, 0, GDBM_WRITER, 0644, 0);
	if (!dbf)
	  return 0;
	
	found = gdbm_delete(dbf, key);
	gdbm_close(dbf);
	
	return (INT64_T) found;
}


/* searches a locally cached group file for a given member */

static INT64_T is_member(char* path, const char* usr_name)
{
        GDBM_FILE dbf;
	datum key;
	int found = 0;

	key.dsize = strlen(usr_name) + 1;
	key.dptr = (char*)malloc(sizeof(char) * key.dsize);
	if (!key.dptr)
	  {
	    return 0;
	  }
	strncpy(key.dptr, usr_name, key.dsize);
	
	dbf = gdbm_open(path, 0, GDBM_READER, 0644, 0);
	if (!dbf)
	  {
	    debug(D_CHIRP, "dbf file %s couldn't be opened\n", path);
	    return 0;
	  }
	
	found = gdbm_exists(dbf, key);
	gdbm_close(dbf);
       
	return (INT64_T) found;
}

/* Determines if an ACL entry is prefaced with "group:" and strips it off */

INT64_T is_group(char* line)
{
        char* s = line;

	if (!s) 
	  {
	    return 0;
	  }

	while (isspace(s[0])) 
	  {
	    s++;
	  }

	if (!strncmp(strtok(s,":"), "group", GROUPNAME_MAX))
	  {
	    strncpy(line, strtok(NULL, " \t\n\r"), GROUPNAME_MAX);   /* copy the rest of the subject starting at line*/
	    return 1;
	  }
	else
	  { 
	    return 0;
	  }
}

/********************************* CACHING FUNCTIONS *************************************/



/* used to find an entry in the cache file index list */

int chirp_group_cmp_cache_files(struct cache_file* a, struct cache_file* b)
{
        return (!strncmp(a->grp_name, b->grp_name, GROUPNAME_MAX));
}

/* used to find an entry in the decision cache list */

int chirp_group_cmp_cache_dec(struct decision* a, struct decision* b)
{
        return ((!strncmp(a->grp_name, b->grp_name, GROUPNAME_MAX)) && (!strncmp(a->usr_name, b->usr_name, USERNAME_MAX)));
}

/* used to find an entry in the policy cache list */

int chirp_group_cmp_cache_policies(struct policy* a, struct policy* b)
{
        return (!strncmp(a->grp_name, b->grp_name, GROUPNAME_MAX));
}

/* this function must set current_group_caching_policy to the current group's policy -- all caching functions will fail if not */

static INT64_T chirp_group_get_caching_policy(const char* grp_name, time_t stoptime)
{
        struct policy* c_policy = NULL;
	struct policy this_policy;
	char hostport[HOSTNAME_MAX], path[CHIRP_PATH_MAX];
	time_t now;

	now = time(0);
	if (!strncmp(current_group_policy.grp_name, grp_name, GROUPNAME_MAX)) /* we've already referenced this host's policy recently */
	  if ((current_group_policy.file_exp >= now) || (current_group_policy.dec_exp >= now))             /* policy is still valid */
	    return current_group_policy.policy;
	
	/* need to check list of all policies to see if we have cached it */
  
	strncpy(this_policy.grp_name, grp_name, HOSTNAME_MAX);
	c_policy = list_find(cache_policy_list, (list_op_t) chirp_group_cmp_cache_policies, &this_policy);
	
	chirp_group_parse_group_name(grp_name, hostport, path);
	if (!c_policy)                                  /* not in the list - need to contact server to get policy */
	  {
	    c_policy = malloc(sizeof(struct policy));
	    if (!c_policy)
	      {
		return -1;
	      }
	    debug(D_CHIRP, "policy not cached\n");
	    if(chirp_reli_get_remote_policy(hostport, grp_name, &c_policy->policy, &c_policy->file_duration, &c_policy->dec_duration, stoptime) < 0)
	      {
		errno = EAGAIN;
		return CHIRP_ERROR_GRP_UNREACHABLE;
	      }
	  }
	else if ((c_policy->file_exp < now) && (c_policy->dec_exp < now))     /* in the list, but policy cache is expired - need new copy*/
	  {
	    c_policy = malloc(sizeof(struct policy));
	    if (!c_policy)
	      {
		return -1;
	      }
	    debug(D_CHIRP, "cached policy expired\n");
	    if (chirp_reli_get_remote_policy(hostport, grp_name, &c_policy->policy, &c_policy->file_duration, &c_policy->dec_duration, stoptime) < 0)
	      return -1;
	  }
    
	if (c_policy)
	  {
	    c_policy->file_exp = now + c_policy->file_duration;  
	    c_policy->dec_exp = now + c_policy->dec_duration;
	    strncpy(c_policy->grp_name, grp_name, GROUPNAME_MAX);
	    list_push_head(cache_policy_list, c_policy);
	    
	    /* need both actual expiration time and duration for both file and decision caches */
	    strncpy(current_group_policy.grp_name, grp_name, GROUPNAME_MAX);
	    current_group_policy.file_exp = now + c_policy->file_duration;  
	    current_group_policy.dec_exp = now + c_policy->dec_duration; 
	    current_group_policy.dec_duration = c_policy->dec_duration;
	    current_group_policy.file_duration = c_policy->file_duration;
	    current_group_policy.policy = c_policy->policy;
	    
	    debug(D_CHIRP, "caching policy %d\n", c_policy->policy);
	    
	    return c_policy->policy;
	  }
	else
	  {
	    return -1;
	  }
 }


/* places a group lookup decision into the list */

static INT64_T chirp_group_cache_decision(const char* grp_name, const char* usr_name, INT64_T result, time_t stoptime)
{
        static struct decision* decision_cache;
	struct hostent* host;
	time_t now;
	char host_name[HOSTNAME_MAX],
	     host_to_chk[HOSTNAME_MAX],
	     virtual_path[CHIRP_PATH_MAX];
	int port = CHIRP_PORT;
	struct policy* p, this_policy;

	decision_cache = malloc(sizeof(struct decision));
	if (!decision_cache)
	  {
	    return -1;
	  }
	
	gethostname(host_name, HOSTNAME_MAX);
	host = gethostbyname(host_name);

	if (sscanf(grp_name, "%[^:]:%d%s", host_to_chk, &port, virtual_path) != 3)
	  sscanf(grp_name, "%[^/]%s", host_to_chk, virtual_path);

	now = time(0);
	if (!strncmp(host->h_name, host_to_chk, HOSTNAME_MAX))     /* group exists on the local host */
	  {
	    decision_cache->expiration = now + local_decision_duration;
	  }
	else
	  {
	    if(!strncmp(current_group_policy.grp_name, grp_name, GROUPNAME_MAX)) /* we have valid remote server's policy cached */
	      {
		strncpy(this_policy.grp_name, grp_name, HOSTNAME_MAX);
		p = list_find(cache_policy_list, (list_op_t) chirp_group_cmp_cache_policies, &this_policy);
		current_group_policy.dec_exp = now + p->dec_duration;
		decision_cache->expiration = current_group_policy.dec_exp;
	      }
	    /*      else   worst case -- we need to remotely verify the caching policy expiration */
	    {
	      chirp_group_get_caching_policy(grp_name, stoptime);   
	      decision_cache->expiration = current_group_policy.dec_exp;
	    }
	  }
	strncpy(decision_cache->grp_name, grp_name, GROUPNAME_MAX);
	strncpy(decision_cache->usr_name, usr_name, USERNAME_MAX);
	decision_cache->result = result;
	list_push_head(dec_cache_list, decision_cache);
	debug(D_CHIRP, "caching lookup decision\n");
	return 1;
}

static INT64_T chirp_group_uncache_decision( struct list *l, const struct decision* value )
{
        struct list_node *n;
	struct decision* data;  

	for(n=l->head;n;n=n->next)
	  {
	    data = n->data;
	    if((!strncmp(data->grp_name, value->grp_name, GROUPNAME_MAX)) && (!strncmp(data->usr_name, value->usr_name, USERNAME_MAX)))
	      {
		debug(D_CHIRP, "uncaching decision\n");
		if( n->next ) n->next->prev = n->prev;
		if( n->prev ) n->prev->next = n->next;
		if( n==l->head ) l->head = n->next;
		if( n==l->tail ) l->tail = n->prev;
		free(n);
		l->size--;
		return 1;
	      }
	  }
	return 0;
}

/* returns the result of a search of the decision cache list */

static INT64_T chirp_group_check_decision_cache( char* grp_name, const char* usr_name, INT64_T* found )
{
        struct decision* decision_cache;
	struct decision entry_to_chk;
	
	strncpy(entry_to_chk.grp_name, grp_name, GROUPNAME_MAX);
	strncpy(entry_to_chk.usr_name, usr_name, USERNAME_MAX);
	
	decision_cache = list_find(dec_cache_list, (list_op_t) chirp_group_cmp_cache_dec, &entry_to_chk);
	
	if (!decision_cache)
	  {
	    return 0;
	  }
	
	if (decision_cache->expiration < time(0))
	  {
	    debug(D_CHIRP, "decision cache has expired\n");
	    chirp_group_uncache_decision(dec_cache_list, decision_cache);
	    return 0;
	  }
	else 
	  {
	    *found = decision_cache->result;
	    return 1;
	  }
	return 0;
}

INT64_T chirp_group_check_cache_time( const char* virtual_path, const time_t cache_mod_time )
{
        char local_path[CHIRP_PATH_MAX];
	struct stat buf;

	sprintf(local_path, "%s/%s", root_path, virtual_path);
	string_collapse_path(local_path, local_path, 1);
	
	if (!stat(local_path, &buf))
	  {
	    if (buf.st_mtime <= cache_mod_time)
	      {
		return 1;
	      }
	  }
	return 0;
}

static INT64_T chirp_group_update_cache( const char* grp_name, time_t stoptime )
{
        struct cache_file file_to_chk;
	struct cache_file *file_found;
	char path[CHIRP_PATH_MAX];
	char hostport[HOSTNAME_MAX];
	struct stat buf;
	
	strncpy(file_to_chk.grp_name, grp_name, GROUPNAME_MAX);
	
	file_found = list_find(cache_file_index, (list_op_t) chirp_group_cmp_cache_files, &file_to_chk);
	
	if (!file_found)
	  {
	    return 0;
	  }
	
	if (file_found->expiration >= time(0))
	  {
	    return 1;
	  }

	sprintf(path, "%s/%s/%s", root_path, CACHE_DIR, file_found->file_name);
	if (stat(path, &buf) < 0)
	  {
	    return 0;
	  }

	chirp_group_parse_group_name(grp_name, hostport, path);
	if (chirp_reli_update_cache(hostport, path, buf.st_mtime, stoptime) >0)
	  {
	    file_found->expiration = current_group_policy.file_exp;
	    current_group_policy.file_exp += current_group_policy.file_duration;
	    return 1;
	  }
	return 0;
}


/* checks for a valid list entry for a cached group file and performs a lookup if it exists */

static INT64_T chirp_group_check_cache( const char* grp_name, const char* usr_name, INT64_T* found, time_t stoptime )
{
        char cache_path[CHIRP_PATH_MAX];
	struct cache_file entry_to_chk;
	struct cache_file* file;
	
	strncpy(entry_to_chk.grp_name, grp_name, GROUPNAME_MAX);
	
	file = list_find(cache_file_index, (list_op_t) chirp_group_cmp_cache_files, &entry_to_chk); 
	
	if (!file) /* no valid cached entry */
	  return 0;
	
	if (!strcmp(file->grp_name, grp_name))
	  {
	    if (file->expiration < time(0))
	      {
		debug(D_CHIRP, "cached group file for %s expired\n", grp_name);
		if (!chirp_group_update_cache(grp_name, stoptime))
		  {
		    return 0;
		  }
	      }
	    else
	      {
		sprintf(cache_path, "%s/%s/%s", root_path, CACHE_DIR, file->file_name);
		debug(D_CHIRP, "lookup on cached group file\n");
		*found = is_member(cache_path, usr_name);
		if (current_group_policy.policy & DECISION)
		  {
		    if (!errno)
		      chirp_group_cache_decision(grp_name, usr_name, *found, stoptime);
		  }
		return 1; /* successful lookup */
	      }
	  }
	return 0;
}

/* retrieves a file from remote server and adds an entry to the cached files list */

static INT64_T chirp_group_get_cache( const char* grp_name, time_t stoptime )
{
        static struct cache_file* new_entry;
	const char *filename;
	char        cache_path[CHIRP_PATH_MAX],
	            host_to_chk[HOSTNAME_MAX],
                    virtual_path[CHIRP_PATH_MAX];
	FILE*       cache;
	INT64_T     port = CHIRP_PORT;
	int fd;

        new_entry=malloc(sizeof(struct cache_file));

	/* create the local cached file */
	chirp_group_create_cache_dir();

	/* create local temp file to store the cached remote group file */
	sprintf(cache_path, "%s/%s/tmpXXXXXX", root_path, CACHE_DIR);
	filename = string_basename(cache_path);
	fd = mkstemp(cache_path);
	cache = fdopen(fd, "w");
	if (!cache)
	  {
	    debug(D_CHIRP, "couldn't open %s\n", cache_path);
	    return 0;
	  }
	
	if (sscanf(grp_name, "%[^:]:%lld%s", host_to_chk, &port, virtual_path) != 3)
	  {
	    sscanf(grp_name, "%[^/]%s", host_to_chk, virtual_path);
	  }
	
	sprintf(host_to_chk, "%s:%lld", host_to_chk, port);
	/* orignal group file has been modified or we don't already have a cached copy, so get one */
	if (chirp_reli_getfile(host_to_chk, virtual_path, cache, stoptime) > 0)
	  {
	    new_entry->expiration = current_group_policy.file_exp;
	    fclose(cache);
	  }
	else
	  {
	    fclose(cache);
	    return 0;
	  }
	
	/* copy the file data into the index and add the index to the list */
	strncpy(new_entry->grp_name, grp_name, GROUPNAME_MAX);
	strncpy(new_entry->file_name, filename, USERNAME_MAX);
	
	list_push_head(cache_file_index, new_entry);

	return 1;
}


INT64_T chirp_group_get_policy_from_db(const char* grp_name, int *cache_file_duration, int *cache_dec_duration)
{
        char path_to_db[CHIRP_PATH_MAX];
	char host[HOSTNAME_MAX];
	char path[CHIRP_PATH_MAX];
	INT64_T port, result = 0;
	GDBM_FILE grp_policy_db;
	datum key, value;
	
	sprintf(path_to_db, "%s/%s/%s", root_path, CACHE_DIR, POLICY_DB);
	
	grp_policy_db = gdbm_open(path_to_db, 0, GDBM_READER, 0644, 0);
	
	if (!grp_policy_db)
	  {
	    return -1;
	  }

	if (sscanf(grp_name, "%[^:]:%lld%s", host, &port, path) != 3)
	  {
	    sscanf(grp_name, "%[^/]%s", host, path);
	  }

	key.dsize = strlen(path) + 1;
	key.dptr = malloc(key.dsize);
	if (!key.dptr)
	  {
	    return -1;
	  }

	strncpy(key.dptr, path, key.dsize);
	value = gdbm_fetch(grp_policy_db, key);

	gdbm_close(grp_policy_db);

	*cache_file_duration = *cache_dec_duration = 0;
	if (value.dsize > 0)
	  {
	    sscanf(value.dptr, "%d %d", cache_file_duration, cache_dec_duration);
	    free(value.dptr);
	    if (*cache_file_duration > 0)
	      {
		result += GROUP_FILE;
	      }
	    if (*cache_dec_duration > 0)
	      {
		result += DECISION;
	      }
	    return result;
	  }
	return 0;
}

/*************************************** GROUP LOOKUP FUNCTIONS ***************************************/

INT64_T chirp_group_do_lookup(char* grp_name, const char* usr_name, time_t stoptime)
{
  
        INT64_T port = CHIRP_PORT, 
                cache_exists = 0, 
                cache_policy = 0,
                found = 0;
	char    this_host[HOSTNAME_MAX],
                host_to_chk[HOSTNAME_MAX],
                full_path[USERNAME_MAX],
                virtual_path[USERNAME_MAX];

	/* determine the name of the current host */
	gethostname(this_host, HOSTNAME_MAX);
	strncpy(this_host, gethostbyname(this_host)->h_name, HOSTNAME_MAX);

	/* check to see if port is specified in ACL entry.  if not, use default */
	if (sscanf(grp_name, "%[^:]:%lld%s", host_to_chk, &port, virtual_path) != 3)
	  {
	    sscanf(grp_name, "%[^/]%s", host_to_chk, virtual_path);
	  }

	/* determine if lookup is local or remote */
	if (!strncmp(host_to_chk, this_host, HOSTNAME_MAX)) 
	  {
	    if (chirp_group_check_decision_cache(grp_name, usr_name, &found))        /* no need to worry about policy on a local host! */
	      {
		return found;
	      }

	    sprintf(full_path, "%s%s", root_path, virtual_path);                     /* perform lookup on group file on local host */
	    debug(D_CHIRP, "lookup on local group file (not cache)");
	    found = is_member(full_path, usr_name);
	    if (!errno)
	      chirp_group_cache_decision(grp_name, usr_name, found, stoptime);
	  }
	else  /* group is defined on remote host */
	  {
	    cache_policy = chirp_group_get_caching_policy(grp_name, stoptime);                 /* determine caching policy for remote group*/
	    if (cache_policy < 0)
	      {
		return -1;
	      }

	    if (current_group_policy.policy & DECISION)                               /* remote server's policy allows decision caching */
	      {
		if (chirp_group_check_decision_cache(grp_name, usr_name, &found))
		  {
		    return found;
		  }
	      }

	    /* no cached decision exists...let's check the group file */
	    cache_exists = chirp_group_check_cache(grp_name, usr_name, &found, stoptime);  /* check for valid local cache and get result */
	    if (!cache_exists)                                                       /* no cache to check, but lookup is remote, so connect to remote server */
	      {                            
		found = chirp_group_remote_lookup(grp_name, usr_name, stoptime);
	      }
	  }
	return found;
}

/* gets the result of a group lookup on a remote server */

INT64_T chirp_group_remote_lookup( char* grp_name, const char* usr_name, time_t stoptime )
{
        char hostport[HOSTNAME_MAX], 
	     path[CHIRP_PATH_MAX];
	INT64_T result;

	if (chirp_group_get_caching_policy(grp_name, stoptime) < 0)                             /* check for expired policy */
	  {
	    errno = EAGAIN;
	    return -1;
	  }
	
	if (current_group_policy.policy & DECISION)                                     /* remote server's policy allows decision caching */
	  {
	    if (chirp_group_check_decision_cache(grp_name, usr_name, &result))
	      {
		return result;
	      }
	  }

	chirp_group_parse_group_name(grp_name, hostport, path);
	if (!(current_group_policy.policy & GROUP_FILE))                                /* policy is to not allow caching of group files */
	  {
	    result = chirp_reli_group_lookup(hostport, grp_name, usr_name, stoptime);
	    if (current_group_policy.policy & DECISION)
	      {
		chirp_group_cache_decision(grp_name, usr_name, result, stoptime);
	      }
	  }
	else                                                                           /* groups may be cached but no local copy exists, so get one and check it */
	  {
	    if (!chirp_group_update_cache(grp_name, stoptime))
	      {
		if (chirp_group_get_cache(grp_name, stoptime))
		  {
		    chirp_group_check_cache(grp_name, usr_name, &result, stoptime);
		  }
		else
		  {
		    result = chirp_reli_group_lookup(hostport, grp_name, usr_name, stoptime);
		  }
	      }
	  }
	return result;
}




/* clears out the cache directory that sometimes becomes cluttered if servers crash */

INT64_T chirp_group_empty_file_cache(const char* path)
{
        DIR* dir;
	struct dirent* entry;
	char cache_dir[CHIRP_PATH_MAX], filename[CHIRP_PATH_MAX];

	sprintf(cache_dir, "%s/%s", path, CACHE_DIR);
	dir = opendir(cache_dir);
	
	if (!dir)
	  {
	    return -1;
	  }

	while((entry = readdir(dir)))
	  {
	    if (strncmp(entry->d_name, ".", strlen(".")) == 0)
	      {
		continue;
	      }
	    if (strncmp(entry->d_name, "..", strlen("..")) == 0)
	      {
		continue;
	      }
	    if (strncmp(entry->d_name, ".__acl", strlen(".__acl")) == 0)
	      {
		continue;
	      }
	    if (strncmp(entry->d_name, POLICY_DB, strlen(POLICY_DB)) == 0)
	      {
		continue;
	      }
	    if(entry->d_name)
	      {
		sprintf(filename, "%s/%s", cache_dir, entry->d_name);
		if (unlink(filename) == -1)
		  {
		    continue;
		  }
	      }
	  }
	closedir(dir);
	return 1;
}

void chirp_group_parse_group_name(const char* grp_name, char* hostport, char* path)
{
        INT64_T port = CHIRP_PORT;

	if (sscanf(grp_name, "%[^:]:%lld%s", hostport, &port, path) != 3)
	  {
	    sscanf(grp_name, "%[^/]%s", hostport, path);
	  }
	
	sprintf(hostport, "%s:%lld", hostport, port);
}
